/**
 * apeman task to setup database.
 * @memberof module:apeman-task-setupdb/lib
 * @function define
 * @param {Apemandb|string} db - Apeman database instance or it's module path.
 * @param {object} [options] - Optional settings.
 * @returns {function} - Defined task function.
 */

'use strict'

const assert = require('assert')
const path = require('path')
const co = require('co')
const findout = require('findout')

const setupMysql = require('./setup/setup_mysql')
const setupSqlite = require('./setup/setup_sqlite')
const setupPostgres = require('./setup/setup_postgres')

/** @lends define */
function define (db, options = {}) {
  /**
   * Defined task.
   * @function task
   * @param {object} context - Apeman task context.
   * @returns {Promise}
   */
  function task (context) {
    if (typeof(db) === 'string') {
      db = findout(db)
    }

    assert.ok(db, "db is required.")

    let env = db.env
    assert.ok(env, "db.env is required.")

    env = _upperKeys(env || {})

    let { logger, verbose } = context

    let dialect = env.DIALECT
    return co(function * () {
      logger.debug(`Setup database...`)
      let { NODE_ENV } = process.env
      if (NODE_ENV) {
        logger.debug(`NODE_ENV: ${NODE_ENV}`)
      }
      logger.debug(`dialect: ${dialect}`)
      if (verbose) {
        logger.trace(env)
      }
      switch (dialect) {
        case 'mysql':
          yield setupMysql(env)
          break
        case 'postgres':
          yield setupPostgres(env)
          break
        case 'sqlite':
          yield setupSqlite(env)
          break
        default:
          throw new Error(`Unknown dialect: ${dialect}`)
      }
      logger.debug(`...Done!`)
    })
  }

  // Description of this task.
  task.$desc = 'Setup database.'
  return task
}

function _upperKeys (obj) {
  let result = {}
  Object.keys(obj).forEach((key) => {
    result[ key.toUpperCase() ] = obj[ key ]
  })
  return result
}

module.exports = define
